@@ -75,5 +75,9 @@ void message_set_autosend(struct message *, unsigned int);
 struct message *message_parse(xmlDocPtr, xmlNsPtr, xmlNodePtr);
 /* Saves a message to an XML file */
 void message_save(struct message *, int, xmlNodePtr);
+/* Loads a message from a file */
+void message_load_binary(struct message *, char *);
+/* Saves a message to a file */
+void message_save_binary(struct message *, char *);
 
 #endif
