@@ -252,6 +252,36 @@ struct block *block_copy(struct block *block, int starttrack, int startline, int
     return NULL;
   }
 
+  /* Bounds checking */
+  if (starttrack < 0)
+    starttrack = 0;
+  if (startline < 0)
+    startline = 0;
+  if (starttrack >= block->tracks)
+    starttrack = block->tracks - 1;
+  if (startline >= block->length)
+    startline = block->length - 1;
+  if (endtrack < 0)
+    endtrack = 0;
+  if (endline < 0)
+    endline = 0;
+  if (endtrack >= block->tracks)
+    endtrack = block->tracks - 1;
+  if (endline >= block->length)
+    endline = block->length - 1;
+
+  /* Make sure start and end are the correct way around */
+  if (starttrack > endtrack) {
+    t = starttrack;
+    starttrack = endtrack;
+    endtrack = t;
+  }
+  if (startline > endline) {
+    l = startline;
+    startline = endline;
+    endline = l;
+  }
+    
   /* Allocate new block */
   newblock = block_alloc(endtrack - starttrack + 1, endline - startline + 1, block->commandpages);
   onl = block->length;
@@ -319,6 +349,36 @@ void block_clear(struct block *block, int starttrack, int startline, int endtrac
     return;
   }
 
+  /* Bounds checking */
+  if (starttrack < 0)
+    starttrack = 0;
+  if (startline < 0)
+    startline = 0;
+  if (starttrack >= block->tracks)
+    starttrack = block->tracks - 1;
+  if (startline >= block->length)
+    startline = block->length - 1;
+  if (endtrack < 0)
+    endtrack = 0;
+  if (endline < 0)
+    endline = 0;
+  if (endtrack >= block->tracks)
+    endtrack = block->tracks - 1;
+  if (endline >= block->length)
+    endline = block->length - 1;
+
+  /* Make sure start and end are the correct way around */
+  if (starttrack > endtrack) {
+    t = starttrack;
+    starttrack = endtrack;
+    endtrack = t;
+  }
+  if (startline > endline) {
+    l = startline;
+    startline = endline;
+    endline = l;
+  }
+    
   nl = block->length;
   nt = block->tracks;
 
@@ -496,22 +556,36 @@ void block_transpose(struct block *block, int instrument, int halfnotes, int sta
     starttrack = block->tracks - 1;
   if (startline >= block->length)
     startline = block->length - 1;
-  if (endtrack < starttrack)
-    endtrack = starttrack;
-  if (endline < startline)
-    endline = startline;
+  if (endtrack < 0)
+    endtrack = 0;
+  if (endline < 0)
+    endline = 0;
   if (endtrack >= block->tracks)
     endtrack = block->tracks - 1;
   if (endline >= block->length)
     endline = block->length - 1;
 
+  /* Make sure start and end are the correct way around */
+  if (starttrack > endtrack) {
+    t = starttrack;
+    starttrack = endtrack;
+    endtrack = t;
+  }
+  if (startline > endline) {
+    l = startline;
+    startline = endline;
+    endline = l;
+  }
+    
   /* Either check for correct instrument or transpose all instruments */
   if (instrument < 0)
     for (l = startline; l <= endline; l++)
       for (t = starttrack; t <= endtrack; t++) {
 	if (block->notes[(l * block->tracks + t) * 2] > 0) {
-	  if (block->notes[(l * block->tracks + t) * 2] + halfnotes < 0 || block->notes[(l * block->tracks + t) * 2] + halfnotes > 127)
+	  if (block->notes[(l * block->tracks + t) * 2] + halfnotes < 1)
 	    block->notes[(l * block->tracks + t) * 2] = 1;
+	  else if (block->notes[(l * block->tracks + t) * 2] + halfnotes > 127)
+	    block->notes[(l * block->tracks + t) * 2] = 127;
 	  else
 	    block->notes[(l * block->tracks + t) * 2] += halfnotes;
 	}
@@ -519,8 +593,10 @@ void block_transpose(struct block *block, int instrument, int halfnotes, int sta
     for (l = startline; l <= endline; l++)
       for (t = starttrack; t <= endtrack; t++) {
 	if (block->notes[(l * block->tracks + t) * 2] > 0 && block->notes[(l * block->tracks + t) * 2 + 1] == instrument + 1) {
-	  if (block->notes[(l * block->tracks + t) * 2] + halfnotes < 0 || block->notes[(l * block->tracks + t) * 2] + halfnotes > 127)
+	  if (block->notes[(l * block->tracks + t) * 2] + halfnotes < 0)
 	    block->notes[(l * block->tracks + t) * 2] = 1;
+	  else if(block->notes[(l * block->tracks + t) * 2] + halfnotes > 127)
+	    block->notes[(l * block->tracks + t) * 2] = 127;
 	  else
 	    block->notes[(l * block->tracks + t) * 2] += halfnotes;
 	}
@@ -530,7 +606,7 @@ void block_transpose(struct block *block, int instrument, int halfnotes, int sta
 /* Expands/shrinks a block or a part of it */
 void block_expandshrink(struct block *block, int factor, int starttrack, int startline, int endtrack, int endline)
 {
-  int l, t;
+  int l, t, p;
 
   if (block == NULL) {
     fprintf(stderr, "block_expandshrink() called with null block\n");
@@ -549,15 +625,27 @@ void block_expandshrink(struct block *block, int factor, int starttrack, int sta
     starttrack = block->tracks - 1;
   if (startline >= block->length)
     startline = block->length - 1;
-  if (endtrack < starttrack)
-    endtrack = starttrack;
-  if (endline < startline)
-    endline = startline;
+  if (endtrack < 0)
+    endtrack = 0;
+  if (endline < 0)
+    endline = 0;
   if (endtrack >= block->tracks)
     endtrack = block->tracks - 1;
   if (endline >= block->length)
     endline = block->length - 1;
 
+  /* Make sure start and end are the correct way around */
+  if (starttrack > endtrack) {
+    t = starttrack;
+    starttrack = endtrack;
+    endtrack = t;
+  }
+  if (startline > endline) {
+    l = startline;
+    startline = endline;
+    endline = l;
+  }
+    
   if (factor < 0) {
     /* Shrink */
     for (l = startline; l <= endline; l++)
@@ -565,11 +653,19 @@ void block_expandshrink(struct block *block, int factor, int starttrack, int sta
 	for (t = starttrack; t <= endtrack; t++) {
 	  block->notes[(l * block->tracks + t) * 2] = block->notes[((startline + (l - startline) * -factor) * block->tracks + t) * 2];
 	  block->notes[(l * block->tracks + t) * 2 + 1] = block->notes[((startline + (l - startline) * -factor) * block->tracks + t) * 2 + 1];
+	  for (p = 0; p < block->commandpages; p++) {
+	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) * -factor) * block->tracks + t) * 2];
+	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) * -factor) * block->tracks + t) * 2 + 1];
+	  }
 	}
       } else {
 	for (t = starttrack; t <= endtrack; t++) {
 	  block->notes[(l * block->tracks + t) * 2] = 0;
 	  block->notes[(l * block->tracks + t) * 2 + 1] = 0;
+	  for (p = 0; p < block->commandpages; p++) {
+	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = 0;
+	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = 0;
+	  }
 	}
       }
   } else {
@@ -579,11 +675,19 @@ void block_expandshrink(struct block *block, int factor, int starttrack, int sta
 	for (t = starttrack; t <= endtrack; t++) {
 	  block->notes[(l * block->tracks + t) * 2] = block->notes[((startline + (l - startline) / factor) * block->tracks + t) * 2];
 	  block->notes[(l * block->tracks + t) * 2 + 1] = block->notes[((startline + (l - startline) / factor) * block->tracks + t) * 2 + 1];
+	  for (p = 0; p < block->commandpages; p++) {
+	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) / factor) * block->tracks + t) * 2];
+	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = block->commands[p * block->tracks * block->length * 2 + ((startline + (l - startline) / factor) * block->tracks + t) * 2 + 1];
+	  }
 	}
       } else {
 	for (t = starttrack; t <= endtrack; t++) {
 	  block->notes[(l * block->tracks + t) * 2] = 0;
 	  block->notes[(l * block->tracks + t) * 2 + 1] = 0;
+	  for (p = 0; p < block->commandpages; p++) {
+	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2] = 0;
+	    block->commands[p * block->tracks * block->length * 2 + (l * block->tracks + t) * 2 + 1] = 0;
+	  }
 	}
       }
   }
@@ -608,15 +712,27 @@ void block_changeinstrument(struct block *block, int from, int to, int swap, int
     starttrack = block->tracks - 1;
   if (startline >= block->length)
     startline = block->length - 1;
-  if (endtrack < starttrack)
-    endtrack = starttrack;
-  if (endline < startline)
-    endline = startline;
+  if (endtrack < 0)
+    endtrack = 0;
+  if (endline < 0)
+    endline = 0;
   if (endtrack >= block->tracks)
     endtrack = block->tracks - 1;
   if (endline >= block->length)
     endline = block->length - 1;
 
+  /* Make sure start and end are the correct way around */
+  if (starttrack > endtrack) {
+    t = starttrack;
+    starttrack = endtrack;
+    endtrack = t;
+  }
+  if (startline > endline) {
+    l = startline;
+    startline = endline;
+    endline = l;
+  }
+    
   if (swap)
     for (l = endline; l >= startline; l--)
       for (t = starttrack; t <= endtrack; t++) {
