@@ -44,7 +44,7 @@ void on_menuitem_file_new_activate(GtkMenuItem * menuitem, gpointer user_data)
 void on_menuitem_file_open_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_open");
+  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_file_open");
 
   if (!GTK_WIDGET_VISIBLE(fs))
     gtk_widget_show(fs);
@@ -56,11 +56,17 @@ void on_menuitem_file_save_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
   int l;
-  
+
+  /* If no filename has been specified open the save as dialog */
+  if (gui->editor->filename == NULL) {
+    on_menuitem_file_saveas_activate(menuitem, user_data);
+    return;
+  }
+
   /* Save in the format determined by the filename extension */
-  l=strlen(gui->editor->filename);
-  if(l>4 && (strcasecmp(gui->editor->filename+l-3, "med")==0 || strcasecmp(gui->editor->filename+l-3, "mmd")==0 || strcasecmp(gui->editor->filename+l-4, "mmd2")==0)) {
-    struct MMD2 *mmd=song_convert(gui->editor->song);
+  l = strlen(gui->editor->filename);
+  if (l > 4 && (strcasecmp(gui->editor->filename + l - 3, "med") == 0 || strcasecmp(gui->editor->filename + l - 3, "mmd") == 0 || strcasecmp(gui->editor->filename + l - 4, "mmd2") == 0)) {
+    struct MMD2 *mmd = song_convert(gui->editor->song);
     MMD2_save(mmd, gui->editor->filename);
     MMD2_free(mmd);
   } else
@@ -70,7 +76,7 @@ void on_menuitem_file_save_activate(GtkMenuItem * menuitem, gpointer user_data)
 void on_menuitem_file_saveas_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_saveas");
+  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_file_saveas");
 
   if (!GTK_WIDGET_VISIBLE(fs))
     gtk_widget_show(fs);
@@ -88,7 +94,13 @@ void on_menuitem_file_print_activate(GtkMenuItem * menuitem, gpointer user_data)
 
 void on_menuitem_file_exit_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
-  gtk_main_quit();
+  struct gui *gui = (struct gui *)user_data;
+  GtkWidget *window = glade_xml_get_widget(gui->xml, "dialog_quit");
+
+  if (!GTK_WIDGET_VISIBLE(window))
+    gtk_widget_show(window);
+  else
+    gtk_widget_hide(window);
 }
 
 void on_menuitem_edit_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
@@ -96,7 +108,7 @@ void on_menuitem_edit_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
   struct gui *gui = (struct gui *)user_data;
   Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
 
-  if(tracker_is_in_selection_mode(tracker))
+  if (tracker_is_in_selection_mode(tracker))
     tracker_mark_selection(tracker, FALSE);
   if (gui->editor->copyarea)
     block_free(gui->editor->copyarea);
@@ -110,12 +122,11 @@ void on_menuitem_edit_copy_activate(GtkMenuItem * menuitem, gpointer user_data)
   struct gui *gui = (struct gui *)user_data;
   Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
 
-  if(tracker_is_in_selection_mode(tracker))
+  if (tracker_is_in_selection_mode(tracker))
     tracker_mark_selection(tracker, FALSE);
   if (gui->editor->copyarea)
     block_free(gui->editor->copyarea);
   gui->editor->copyarea = block_copy(gui->editor->song->blocks[gui->editor->block], tracker->sel_start_ch, tracker->sel_start_row, tracker->sel_end_ch, tracker->sel_end_row);
-  tracker_clear_mark_selection(tracker);
 }
 
 void on_menuitem_edit_paste_activate(GtkMenuItem * menuitem, gpointer user_data)
@@ -133,7 +144,6 @@ void on_menuitem_edit_clear_activate(GtkMenuItem * menuitem, gpointer user_data)
   Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
 
   block_clear(gui->editor->song->blocks[gui->editor->block], tracker->sel_start_ch, tracker->sel_start_row, tracker->sel_end_ch, tracker->sel_end_row);
-  tracker_clear_mark_selection(tracker);
 }
 
 void on_menuitem_edit_transpose_activate(GtkMenuItem * menuitem, gpointer user_data)
@@ -496,7 +506,7 @@ void on_menuitem_track_changeinstrument_activate(GtkMenuItem * menuitem, gpointe
 void on_menuitem_song_messagelist_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  GtkWidget *window = glade_xml_get_widget(gui->xml, "dialog_message");
+  GtkWidget *window = glade_xml_get_widget(gui->xml, "dialog_messagelist");
 
   if (!GTK_WIDGET_VISIBLE(window))
     gtk_widget_show(window);
@@ -734,8 +744,13 @@ void on_button_message_delete_clicked(GtkButton * button, gpointer user_data)
   struct gui *gui = (struct gui *)user_data;
   if (gui->messagelist_message != -1) {
     song_delete_message(gui->editor->song, gui->messagelist_message);
+
+    if (gui->messagelist_message >= gui->editor->song->nummessages)
+      gui->messagelist_message = gui->editor->song->nummessages - 1;
+
     gui_messagelist_refresh(gui, TRUE);
   }
+
 }
 
 void on_button_message_insertnew_clicked(GtkButton * button, gpointer user_data)
@@ -760,11 +775,18 @@ void on_button_message_receive_clicked(GtkButton * button, gpointer user_data)
   struct gui *gui = (struct gui *)user_data;
   if (gui->messagelist_message != -1) {
     midi_read_system_exclusive(gui->editor->player->midi, gui->editor->song->messages[gui->messagelist_message], gui->messagelist_autostop);
+    gui_messagelist_refresh(gui, FALSE);
   }
 }
 
 void on_checkbutton_message_autostop_toggled(GtkToggleButton * togglebutton, gpointer user_data)
 {
+  struct gui *gui = (struct gui *)user_data;
+
+  if (gtk_toggle_button_get_active(togglebutton))
+    gui->messagelist_autostop = 1;
+  else
+    gui->messagelist_autostop = 0;
 
 }
 
@@ -991,7 +1013,6 @@ void on_spinbutton_blocklist_commandpages_changed(GtkEditable * editable, gpoint
     gui_tracker_refresh(gui);
   }
   gui_info_refresh(gui);
-  gui_window_title_refresh(gui);
 }
 
 void on_button_blocklist_insertnew_clicked(GtkButton * button, gpointer user_data)
@@ -1044,6 +1065,7 @@ gboolean on_entry_blocklist_focus_out_event(GtkWidget * widget, GdkEventFocus *
 
   gui_blocklist_refresh(gui, TRUE);
   gui_playseq_refresh(gui, TRUE);
+  gui_info_refresh(gui);
 
   return FALSE;
 }
@@ -1095,78 +1117,72 @@ gboolean on_entry_instrumentproperties_name_focus_out_event(GtkWidget * widget,
   return FALSE;
 }
 
-void on_button_message_close_clicked(GtkButton * button, gpointer user_data)
+void on_button_messagelist_close_clicked(GtkButton * button, gpointer user_data)
 {
   on_menuitem_song_messagelist_activate(NULL, user_data);
 }
 
-void on_button_saveas_ok_clicked(GtkButton * button, gpointer user_data)
+void on_button_file_saveas_ok_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_saveas");
+  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_file_saveas");
   gchar *path;
   int l;
-  
+
   /* Store the filename */
   if (gui->editor->filename != NULL)
     free(gui->editor->filename);
   gui->editor->filename = strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
 
   /* Save in the format determined by the filename extension */
-  l=strlen(gui->editor->filename);
-  if(l>4 && (strcasecmp(gui->editor->filename+l-3, "med")==0 || strcasecmp(gui->editor->filename+l-3, "mmd")==0 || strcasecmp(gui->editor->filename+l-4, "mmd2")==0)) {
-    struct MMD2 *mmd=song_convert(gui->editor->song);
+  l = strlen(gui->editor->filename);
+  if (l > 4 && (strcasecmp(gui->editor->filename + l - 3, "med") == 0 || strcasecmp(gui->editor->filename + l - 3, "mmd") == 0 || strcasecmp(gui->editor->filename + l - 4, "mmd2") == 0)) {
+    struct MMD2 *mmd = song_convert(gui->editor->song);
     MMD2_save(mmd, gui->editor->filename);
     MMD2_free(mmd);
   } else
     song_save(gui->editor->song, gui->editor->filename);
 
   /* Store the path in GConf for further use */
-  path=g_path_get_dirname(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
-  path=g_realloc(path, strlen(path)+2);
-  path[strlen(path)+1]=0;
-  path[strlen(path)]='/';
-  gconf_client_set_string(gui->editor->gconfclient, "/apps/tutka/saveaspath", path, NULL);
+  path = g_path_get_dirname(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+  path = g_realloc(path, strlen(path) + 2);
+  path[strlen(path) + 1] = 0;
+  path[strlen(path)] = '/';
+  gconf_client_set_string(gui->editor->gconfclient, "/apps/tutka/songpath", path, NULL);
   g_free(path);
 
   /* ...and close the file selector */
   gtk_widget_hide(fs);
 }
 
-void on_button_open_ok_clicked(GtkButton * button, gpointer user_data)
+void on_button_file_open_ok_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_open");
+  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_file_open");
   gchar *path;
-  
+
   editor_song_open(gui->editor, (char *)
 		   gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
 
   /* Store the path in GConf for further use */
-  path=g_path_get_dirname(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
-  path=g_realloc(path, strlen(path)+2);
-  path[strlen(path)+1]=0;
-  path[strlen(path)]='/';
-  gconf_client_set_string(gui->editor->gconfclient, "/apps/tutka/openpath", path, NULL);
+  path = g_path_get_dirname(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+  path = g_realloc(path, strlen(path) + 2);
+  path[strlen(path) + 1] = 0;
+  path[strlen(path)] = '/';
+  gconf_client_set_string(gui->editor->gconfclient, "/apps/tutka/songpath", path, NULL);
   g_free(path);
 
   gtk_widget_hide(fs);
 }
 
-void on_button_open_cancel_clicked(GtkButton * button, gpointer user_data)
+void on_button_file_open_cancel_clicked(GtkButton * button, gpointer user_data)
 {
-  struct gui *gui = (struct gui *)user_data;
-  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_open");
-
-  gtk_widget_hide(fs);
+  on_menuitem_file_open_activate(NULL, user_data); 
 }
 
-void on_button_saveas_cancel_clicked(GtkButton * button, gpointer user_data)
+void on_button_file_saveas_cancel_clicked(GtkButton * button, gpointer user_data)
 {
-  struct gui *gui = (struct gui *)user_data;
-  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_saveas");
-
-  gtk_widget_hide(fs);
+  on_menuitem_file_saveas_activate(NULL, user_data); 
 }
 
 void on_spinbutton_message_length_changed(GtkEditable * editable, gpointer user_data)
@@ -1492,10 +1508,10 @@ void on_menuitem_preferences_nanosleep_activate(GtkMenuItem * menuitem, gpointer
 gboolean on_entry_preferences_midi_device_focus_out_event(GtkWidget * widget, GdkEventFocus * event, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-    
+
   gconf_client_set_string(gui->editor->gconfclient, "/apps/tutka/mididevice", gtk_entry_get_text(GTK_ENTRY(widget)), NULL);
 
-  return FALSE;  
+  return FALSE;
 }
 
 void on_button_preferences_close_clicked(GtkButton * button, gpointer user_data)
@@ -1503,8 +1519,185 @@ void on_button_preferences_close_clicked(GtkButton * button, gpointer user_data)
   on_menuitem_settings_preferences_activate(NULL, user_data);
 }
 
-void gconf_notify_function(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data) {
+void gconf_notify_function(GConfClient * client, guint cnxn_id, GConfEntry * entry, gpointer user_data)
+{
   struct gui *gui = (struct gui *)user_data;
 
   gui_preferences_refresh(gui);
 }
+
+void on_button_message_load_clicked(GtkButton * button, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_message_load");
+
+  if (!GTK_WIDGET_VISIBLE(fs))
+    gtk_widget_show(fs);
+  else
+    gtk_widget_hide(fs);
+}
+
+void on_button_message_save_clicked(GtkButton * button, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_message_save");
+
+  if (!GTK_WIDGET_VISIBLE(fs))
+    gtk_widget_show(fs);
+  else
+    gtk_widget_hide(fs);
+}
+
+void on_button_message_save_ok_clicked(GtkButton * button, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_message_save");
+  gchar *path;
+
+  /* Save the message */
+  message_save_binary(gui->editor->song->messages[gui->messagelist_message], (char *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+
+  /* Store the path in GConf for further use */
+  path = g_path_get_dirname(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+  path = g_realloc(path, strlen(path) + 2);
+  path[strlen(path) + 1] = 0;
+  path[strlen(path)] = '/';
+  gconf_client_set_string(gui->editor->gconfclient, "/apps/tutka/messagepath", path, NULL);
+  g_free(path);
+
+  /* ...and close the file selector */
+  gtk_widget_hide(fs);
+
+  /* Refresh the GUI */
+  gui_messagelist_refresh(gui, FALSE);
+}
+
+void on_button_message_load_ok_clicked(GtkButton * button, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  GtkWidget *fs = glade_xml_get_widget(gui->xml, "fileselection_message_load");
+  gchar *path;
+
+  /* Load the message */
+  message_load_binary(gui->editor->song->messages[gui->messagelist_message], (char *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+
+  /* Store the path in GConf for further use */
+  path = g_path_get_dirname(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+  path = g_realloc(path, strlen(path) + 2);
+  path[strlen(path) + 1] = 0;
+  path[strlen(path)] = '/';
+  gconf_client_set_string(gui->editor->gconfclient, "/apps/tutka/messagepath", path, NULL);
+  g_free(path);
+
+  /* ...and close the file selector */
+  gtk_widget_hide(fs);
+
+  /* Refresh the GUI */
+  gui_messagelist_refresh(gui, FALSE);
+}
+
+void on_button_message_load_cancel_clicked(GtkButton * button, gpointer user_data)
+{
+  on_button_message_load_clicked(NULL, user_data);
+}
+
+void on_button_message_save_cancel_clicked(GtkButton * button, gpointer user_data)
+{
+  on_button_message_save_clicked(NULL, user_data);
+}
+
+void on_button_quit_quit_clicked(GtkButton * button, gpointer user_data)
+{
+  gtk_main_quit();
+}
+
+void on_button_quit_cancel_clicked(GtkButton * button, gpointer user_data)
+{
+  on_menuitem_file_exit_activate(NULL, user_data);
+}
+
+void on_window_main_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_menuitem_file_exit_activate(NULL, user_data); 
+}
+
+void on_fileselection_file_saveas_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_file_saveas_cancel_clicked(NULL, user_data); 
+}
+
+void on_fileselection_file_open_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_file_open_cancel_clicked(NULL, user_data);
+}
+
+void on_fileselection_message_save_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_message_save_cancel_clicked(NULL, user_data);
+}
+
+void on_fileselection_message_load_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_message_load_cancel_clicked(NULL, user_data);
+}
+
+void on_dialog_transpose_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_transpose_close_clicked(NULL, user_data);
+}
+
+void on_dialog_songproperties_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_songproperties_close_clicked(NULL, user_data);
+}
+
+void on_dialog_instrumentproperties_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_properties_clicked(NULL, user_data);
+}
+
+void on_dialog_blocklist_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_block_clicked(NULL, user_data);
+}
+
+void on_dialog_playseq_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_position_clicked(NULL, user_data);
+}
+
+void on_dialog_sectionlist_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_section_clicked(NULL, user_data);
+}
+
+void on_dialog_messagelist_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_messagelist_close_clicked(NULL, user_data);
+}
+
+void on_dialog_trackvolumes_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_trackvolumes_close_clicked(NULL, user_data);
+}
+
+void on_dialog_changeinstrument_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_changeinstrument_close_clicked(NULL, user_data);
+}
+
+void on_dialog_expandshrink_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_expandshrink_close_clicked(NULL, user_data);
+}
+
+void on_dialog_preferences_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_preferences_close_clicked(NULL, user_data);
+}
+
+void on_dialog_quit_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  on_button_quit_cancel_clicked(NULL, user_data);
+}
+
