@@ -179,7 +179,7 @@ void editor_set_section(struct editor *editor, int section)
     /* If called from GUI set player position, always refresh GUI */
     if (g_thread_self() == editor->thread)
       player_set_section(editor->player, section);
-    gui_set_refresh(editor->gui, GUI_REFRESH_SECTIONLIST | GUI_REFRESH_INFO | GUI_REFRESH_WINDOW_TITLE);
+    gui_set_refresh(editor->gui, GUI_REFRESH_SECTIONLIST | GUI_REFRESH_INFO);
   }
 }
 
@@ -203,7 +203,7 @@ void editor_set_playseq(struct editor *editor, int playseq)
     /* If called from GUI set player position, always refresh GUI */
     if (g_thread_self() == editor->thread)
       player_set_playseq(editor->player, playseq);
-    gui_set_refresh(editor->gui, GUI_REFRESH_PLAYSEQ | GUI_REFRESH_INFO | GUI_REFRESH_WINDOW_TITLE);
+    gui_set_refresh(editor->gui, GUI_REFRESH_PLAYSEQ | GUI_REFRESH_INFO);
   }
 }
 
@@ -253,7 +253,7 @@ void editor_set_block(struct editor *editor, int block)
     /* If called from GUI set player position, always refresh GUI */
     if (g_thread_self() == editor->thread)
       player_set_block(editor->player, block);
-    gui_set_refresh(editor->gui, GUI_REFRESH_BLOCKLIST | GUI_REFRESH_INFO | GUI_REFRESH_WINDOW_TITLE | GUI_REFRESH_TRACKER);
+    gui_set_refresh(editor->gui, GUI_REFRESH_BLOCKLIST | GUI_REFRESH_INFO | GUI_REFRESH_TRACKER);
   }
 }
 
@@ -324,7 +324,7 @@ void editor_set_tempo(struct editor *editor)
     return;
   }
 
-  gui_set_refresh(editor->gui, GUI_REFRESH_WINDOW_TITLE);
+  gui_set_refresh(editor->gui, GUI_REFRESH_TEMPO);
 }
 
 /* Refreshes playseq from section and block from position */
@@ -356,7 +356,7 @@ void editor_refresh_playseq_and_block(struct editor *editor)
   /* If called from GUI set player position, always refresh GUI */
   if (g_thread_self() == editor->thread)
     player_set_playseq(editor->player, editor->playseq);
-  gui_set_refresh(editor->gui, GUI_REFRESH_PLAYSEQ | GUI_REFRESH_BLOCKLIST | GUI_REFRESH_INFO | GUI_REFRESH_WINDOW_TITLE);
+  gui_set_refresh(editor->gui, GUI_REFRESH_PLAYSEQ | GUI_REFRESH_BLOCKLIST | GUI_REFRESH_INFO);
 }
 
 /* Reads MIDI input */
