@@ -142,11 +142,15 @@ struct gui *gui_open(struct editor *editor, char *filename)
   gui->editor->octave = 3;
 
   /* Set default path in file dialogs */
-  if(gconf_client_get_string(editor->gconfclient, "/apps/tutka/openpath", NULL) != NULL)
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_open")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/openpath", NULL));
-  if(gconf_client_get_string(editor->gconfclient, "/apps/tutka/saveaspath", NULL) != NULL)
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_saveas")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/saveaspath", NULL));
-  
+  if(gconf_client_get_string(editor->gconfclient, "/apps/tutka/songpath", NULL) != NULL) {
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_file_open")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/songpath", NULL));
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_file_saveas")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/songpath", NULL));
+  }
+  if(gconf_client_get_string(editor->gconfclient, "/apps/tutka/messagepath", NULL) != NULL) {
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_message_load")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/messagepath", NULL));
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(glade_xml_get_widget(gui->xml, "fileselection_message_save")), gconf_client_get_string(editor->gconfclient, "/apps/tutka/messagepath", NULL));
+  }
+
   return gui;
 }
 
@@ -178,6 +182,7 @@ void gui_refresh_all(struct gui *gui)
   gui_messagelist_refresh(gui, TRUE);
   gui_trackvolumes_refresh(gui);
   gui_song_refresh(gui);
+  gui_window_title_refresh(gui);
 
   /* Show the main window */
   gtk_widget_show(gui->window_main);
@@ -397,37 +402,20 @@ void gui_instrument_refresh(struct gui *gui, gboolean refresh_spin)
 
 void gui_tempo_refresh(struct gui *gui)
 {
-  int l = 46 + strlen(gui->editor->song->name) + intlen(gui->editor->block + 1) + intlen(gui->editor->song->numblocks) + intlen(1) + intlen(gui->editor->song->blocks[gui->editor->block]->commandpages) + intlen(gui->editor->song->tempo) + intlen(gui->editor->song->ticksperline) + 1;
-  char *title = (char *)calloc(l, sizeof(char));
-
   /* Refresh the song properties window widgets */
   gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_tempo"))), gui->editor->song->tempo);
   gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(glade_xml_get_widget(gui->xml, "scale_tpl"))), gui->editor->song->ticksperline);
-
-  /* Refresh the main window widgets if the window exists */
-  if (gui->window_main != NULL && GTK_IS_WINDOW(gui->window_main)) {
-    snprintf(title, l, "Tutka: %s - Block %d/%d - Cmd Page %d/%d - Tempo %d - TPL %d", gui->editor->song->name, gui->editor->block + 1, gui->editor->song->numblocks, 1, gui->editor->song->blocks[gui->editor->block]->commandpages, gui->editor->song->tempo, gui->editor->song->ticksperline);
-    gtk_window_set_title(GTK_WINDOW(gui->window_main), title);
-  }
-
-  free(title);
 }
 
 void gui_window_title_refresh(struct gui *gui)
 {
-  int l = 46 + strlen(gui->editor->song->name) + intlen(gui->editor->block + 1) + intlen(gui->editor->song->numblocks) + intlen(1) + intlen(gui->editor->song->blocks[gui->editor->block]->commandpages) + intlen(gui->editor->song->tempo) + intlen(gui->editor->song->ticksperline) + 1;
-  char *title = (char *)calloc(l, sizeof(char));
-
-  /* Window title bar */
-  snprintf(title, l, "Tutka: %s - Block %d/%d - Cmd Page %d/%d - Tempo %d - TPL %d", gui->editor->song->name, gui->editor->block + 1, gui->editor->song->numblocks, gui->editor->cmdpage + 1, gui->editor->song->blocks[gui->editor->block]->commandpages, gui->editor->song->tempo, gui->editor->song->ticksperline);
-  gtk_window_set_title(GTK_WINDOW(gui->window_main), title);
-  free(title);
+  gtk_window_set_title(GTK_WINDOW(gui->window_main), gui->editor->song->name);
 }
 
 void gui_statusbar_refresh(struct gui *gui)
 {
-  GtkLabel *status = GTK_LABEL(glade_xml_get_widget(gui->xml,
-						    "label_status"));
+  GtkLabel *status = GTK_LABEL(glade_xml_get_widget(gui->xml, "label_status"));
+  
   /* Status bar */
   switch (gui->editor->player->mode) {
   case MODE_PLAY_SONG:
@@ -444,18 +432,34 @@ void gui_statusbar_refresh(struct gui *gui)
 
 void gui_info_refresh(struct gui *gui)
 {
-  char section[9 + 8], playseq[10 + 8], block[7 + 10 + 32];
+  char *section, *playseq, *block;
+  int sectionlength=8+intlen(gui->editor->section + 1)+1+intlen(gui->editor->song->numsections)+1;
+  int playseqlength=9+intlen(gui->editor->position + 1)+1+intlen(gui->editor->song->playseqs[gui->editor->playseq]->length)+1;
+  int blocklength;
+  
+  if (gui->editor->song->blocks[gui->editor->block]->name != NULL)
+    blocklength=6+intlen(gui->editor->block + 1)+1+intlen(gui->editor->song->numblocks)+15+intlen(gui->editor->cmdpage + 1)+1+intlen(gui->editor->song->blocks[gui->editor->block]->commandpages)+3+strlen(gui->editor->song->blocks[gui->editor->block]->name)+1;
+  else
+    blocklength=6+intlen(gui->editor->block + 1)+1+intlen(gui->editor->song->numblocks)+15+intlen(gui->editor->cmdpage + 1)+1+intlen(gui->editor->song->blocks[gui->editor->block]->commandpages)+2;
+
+  section=malloc(sectionlength);
+  playseq=malloc(playseqlength);
+  block=malloc(blocklength);
 
   /* Refresh the main window widgets */
-  snprintf(section, 8 + 8, "Section %d/%d", gui->editor->section + 1, gui->editor->song->numsections);
-  snprintf(playseq, 9 + 8, "Position %d/%d", gui->editor->position + 1, gui->editor->song->playseqs[gui->editor->playseq]->length);
+  snprintf(section, sectionlength, "Section %d/%d", gui->editor->section + 1, gui->editor->song->numsections);
+  snprintf(playseq, playseqlength, "Position %d/%d", gui->editor->position + 1, gui->editor->song->playseqs[gui->editor->playseq]->length);
   if (gui->editor->song->blocks[gui->editor->block]->name != NULL)
-    snprintf(block, 6 + 10 + 32, "Block %d/%d: %s", gui->editor->block + 1, gui->editor->song->numblocks, gui->editor->song->blocks[gui->editor->block]->name);
+    snprintf(block, blocklength, "Block %d/%d (Command Page %d/%d): %s", gui->editor->block + 1, gui->editor->song->numblocks, gui->editor->cmdpage + 1, gui->editor->song->blocks[gui->editor->block]->commandpages, gui->editor->song->blocks[gui->editor->block]->name);
   else
-    snprintf(block, 6 + 10 + 32, "Block %d/%d: ", gui->editor->block + 1, gui->editor->song->numblocks);
+    snprintf(block, blocklength, "Block %d/%d (Command Page %d/%d)", gui->editor->block + 1, gui->editor->song->numblocks, gui->editor->cmdpage + 1, gui->editor->song->blocks[gui->editor->block]->commandpages);
   gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml, "label_section")), section);
   gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml, "label_position")), playseq);
   gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(gui->xml, "label_block")), block);
+
+  free(section);
+  free(playseq);
+  free(block);
 }
 
 void gui_timer_refresh(struct gui *gui, unsigned int seconds)
@@ -670,25 +674,36 @@ void gui_messagelist_refresh(struct gui *gui, gboolean recreate)
     }
   }
 
-  /* Move back to the previously selected Message and make widgets sensitive */
+  /* Move back to the previously selected message and make widgets sensitive */
   if (gui->messagelist_message != -1) {
     gtk_tree_selection_select_path(gtk_tree_view_get_selection(GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_messagelist"))), gtk_tree_path_new_from_string(oldmessage));
 
-    /* Show the number of the selected Message message */
+    /* Show the number of the selected message */
     snprintf(message, 6 + 8, "Message %d/%d", gui->messagelist_message + 1, gui->editor->song->nummessages);
-    gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "frame_message_controls")), message);
+    gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "frame_messagelist_controls")), message);
+
+    /* Display the properties of the message */
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(gui->xml, "spinbutton_message_length")), gui->editor->song->messages[gui->messagelist_message]->length);
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "table_message1"), TRUE);
+    if(gui->editor->song->messages[gui->messagelist_message]->name != NULL)
+      gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_message_name")), gui->editor->song->messages[gui->messagelist_message]->name);
+    else
+      gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_message_name")), "");
+
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "table_messagelist_controls_1"), TRUE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_send"), TRUE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_receive"), TRUE);
-    /* gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_message_autostop"), TRUE); */
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_load"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_save"), TRUE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_message_autostop"), TRUE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_delete"), TRUE);
   } else {
-    gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "frame_message_controls")), "Message 0/0");
-    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "table_message1"), FALSE);
+    gtk_frame_set_label(GTK_FRAME(glade_xml_get_widget(gui->xml, "frame_messagelist_controls")), "Message 0/0");
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "table_messagelist_controls_1"), FALSE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_send"), FALSE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_receive"), FALSE);
-    /* gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_message_autostop"), FALSE); */
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_load"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_save"), FALSE);
+    gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "checkbutton_message_autostop"), FALSE);
     gtk_widget_set_sensitive(glade_xml_get_widget(gui->xml, "button_message_delete"), FALSE);
   }
 }
@@ -697,33 +712,37 @@ void gui_trackvolumes_refresh(struct gui *gui)
 {
   /* Refresh the track volumes window widgets */
   int i;
-  char track[4];
+  char track[10];
   GtkWidget *table = glade_xml_get_widget(gui->xml, "table_trackvolumes");
 
   /* Destroy all children */
   gtk_container_foreach(GTK_CONTAINER(table), (GtkCallback) gtk_widget_destroy, NULL);
 
-  gtk_table_resize(GTK_TABLE(table), 2, gui->editor->song->maxtracks);
+  gtk_table_resize(GTK_TABLE(table), 3, gui->editor->song->maxtracks);
 
   /* Create widgets for each channel */
   for (i = 0; i < gui->editor->song->maxtracks; i++) {
     GtkWidget *w, *x;
 
+    /* Track numbers */
+    snprintf(track, 10, "Track %d", i + 1);
+    x = gtk_label_new(track);
+    gtk_table_attach(GTK_TABLE(table), x, i, i + 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 2, 4);
+
     /* Volume sliders */
-    w = (GtkWidget *) gtk_adjustment_new((gui->editor->song->trackvolumes[i]) & 127, 0, 127, 1, 1, 0);
+    w = (GtkWidget *) gtk_adjustment_new((gui->editor->song->trackvolumes[i]) & 127, 0, 127, 1, 8, 0);
     g_signal_connect(G_OBJECT(w), "value_changed", G_CALLBACK(on_adjustment_trackvolumes_changed), (gpointer) & gui->editor->song->trackvolumes[i]);
     x = gtk_vscale_new(GTK_ADJUSTMENT(w));
     gtk_range_set_inverted(GTK_RANGE(x), TRUE);
     gtk_scale_set_digits(GTK_SCALE(x), 0);
-    gtk_table_attach(GTK_TABLE(table), x, i, i + 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 4);
+    gtk_table_attach(GTK_TABLE(table), x, i, i + 1, 1, 2, GTK_EXPAND, GTK_EXPAND | GTK_FILL, 2, 4);
 
     /* Mute buttons */
-    snprintf(track, 4, "%d", i + 1);
-    w = gtk_check_button_new_with_label(track);
+    w = gtk_check_button_new_with_label("Mute");
     if (gui->editor->song->trackvolumes[i] > 127)
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
     g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(on_togglebutton_trackvolumes_toggled), (gpointer) & gui->editor->song->trackvolumes[i]);
-    gtk_table_attach(GTK_TABLE(table), w, i, i + 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 4, 4);
+    gtk_table_attach(GTK_TABLE(table), w, i, i + 1, 2, 3, GTK_EXPAND, GTK_FILL, 2, 4);
   }
 
   gtk_widget_show_all(table);
