@@ -103,9 +103,31 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	case GDK_ISO_Left_Tab:
 	  /* CTRL-Shift-Tab: Next command page */
 	  editor_set_commandpage(editor, editor->cmdpage+1);
-	  gui_window_title_refresh(gui);  
+	  gui_info_refresh(gui);  
 	  handled=TRUE;
 	  break;
+	case GDK_Delete:
+	  {
+	    int i;
+
+	    /* Delete command and refresh */
+	    for(i=0; i<song->blocks[editor->block]->commandpages; i++)
+	      block_set_command_full(song->blocks[editor->block], editor->line,
+				     tracker->cursor_ch, i, 0, 0);
+	    tracker_redraw_row(tracker, editor->line);
+	    
+	    if(editor->chord) {
+	      /* Chord mode: Go to next channel */
+	      tracker_step_cursor_channel(tracker, 1);
+	      if(event->keyval!=chord_previous_keyval) {
+		editor->chordstatus++;
+		chord_previous_keyval=event->keyval;
+	      }
+	    } else
+	      editor_set_line(editor, editor->line+editor->space);
+	    handled=TRUE;
+	    break;
+	  }
 	case '0':
 	case '1':
 	case '2':
@@ -145,22 +167,65 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	    tracker_step_cursor_channel(tracker, -1);
 	  handled=TRUE;
 	  break;
+	case GDK_Delete:
+	  {
+	    int i;
+
+	    /* Delete note+command and refresh */
+	    block_set_note(song->blocks[editor->block], editor->line,
+			   tracker->cursor_ch, 0, 0, 0);
+	    for(i=0; i<song->blocks[editor->block]->commandpages; i++)
+	      block_set_command_full(song->blocks[editor->block], editor->line,
+				     tracker->cursor_ch, i, 0, 0);
+	    tracker_redraw_row(tracker, editor->line);
+	    
+	    if(editor->chord) {
+	      /* Chord mode: Go to next channel */
+	      tracker_step_cursor_channel(tracker, 1);
+	      if(event->keyval!=chord_previous_keyval) {
+		editor->chordstatus++;
+		chord_previous_keyval=event->keyval;
+	      }
+	    } else
+	      editor_set_line(editor, editor->line+editor->space);
+	    handled=TRUE;
+	    break;
+	  }
 	case GDK_BackSpace:
 	  /* Shift-Backspace: Insert row */
 	  if(editor->edit) {
 	    int i;
-	    struct block *b=block_copy(song->blocks[editor->block],
-				       tracker->cursor_ch, tracker->patpos,
-				       tracker->cursor_ch,
-				       song->blocks[editor->block]->length-1);
-	    block_paste(song->blocks[editor->block], b,
-			tracker->cursor_ch, tracker->patpos+1);
-	    block_set_note(song->blocks[editor->block],
-			   tracker->patpos, tracker->cursor_ch, 0, 0, 0);
-	    for(i=0; i<song->blocks[editor->block]->commandpages; i++)
-	      block_set_command_full(song->blocks[editor->block],
-				     tracker->patpos, tracker->cursor_ch,
-				     i, 0, 0);
+	    struct block *b;
+	    if(alt) {
+	      int j;
+	      b=block_copy(song->blocks[editor->block],
+			   0, tracker->patpos,
+			   song->blocks[editor->block]->tracks-1,
+			   song->blocks[editor->block]->length-1);
+	      block_paste(song->blocks[editor->block], b,
+			  0, tracker->patpos+1);
+	      for(j=0; j<song->blocks[editor->block]->tracks; j++) {
+		block_set_note(song->blocks[editor->block],
+			       tracker->patpos, j, 0, 0, 0);
+		for(i=0; i<song->blocks[editor->block]->commandpages; i++)
+		  block_set_command_full(song->blocks[editor->block],
+					 tracker->patpos, j,
+					 i, 0, 0);
+	      }
+	    } else {
+	      b=block_copy(song->blocks[editor->block],
+			   tracker->cursor_ch, tracker->patpos,
+			   tracker->cursor_ch,
+			   song->blocks[editor->block]->length-1);
+	      block_paste(song->blocks[editor->block], b,
+			  tracker->cursor_ch, tracker->patpos+1);
+	      block_set_note(song->blocks[editor->block],
+			     tracker->patpos, tracker->cursor_ch, 0, 0, 0);
+	      for(i=0; i<song->blocks[editor->block]->commandpages; i++)
+		block_set_command_full(song->blocks[editor->block],
+				       tracker->patpos, tracker->cursor_ch,
+				       i, 0, 0);
+	    }
 	    block_free(b);
 	    tracker_redraw(tracker);
 	  }
@@ -204,6 +269,29 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  }
 	  break;
 	}
+	case GDK_BackSpace:
+	  {
+	    /* Backspace: delete line */
+	    int i, j;
+	    struct block *b=block_copy(song->blocks[editor->block],
+				       0, tracker->patpos+1,
+				       song->blocks[editor->block]->tracks-1,
+				       song->blocks[editor->block]->length-1);
+	    block_paste(song->blocks[editor->block], b, 0, tracker->patpos);
+	    block_free(b);
+	    for(i=0; i<song->blocks[editor->block]->tracks; i++) {
+	      block_set_note(song->blocks[editor->block],
+			     song->blocks[editor->block]->length-1,
+			     i, 0, 0, 0);
+	      for(j=0; j<song->blocks[editor->block]->commandpages; j++)
+		block_set_command_full(song->blocks[editor->block],
+				       song->blocks[editor->block]->length-1,
+				       i, j, 0, 0);
+	    }
+	    tracker_redraw(tracker);
+	  }
+	  handled=TRUE;
+	  break;
 	default:
 	  break;
 	}
@@ -308,31 +396,15 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	    case 'p':
 	      data=29;
 	      break;
+	    case 229:
+	      data=30;
+	      break;
+	    case 180:
+	      data=31;
+	      break;
 	    case GDK_Delete:
 	      data=0;
 	      break;
-	    case GDK_BackSpace:
-	      {
-		/* Backspace: delete line */
-		struct block *b=block_copy(song->blocks[editor->block],
-					   tracker->cursor_ch,
-					   tracker->patpos+1,
-					   tracker->cursor_ch,
-					   song->blocks[editor->block]->length-1);
-		block_paste(song->blocks[editor->block], b,
-			    tracker->cursor_ch, tracker->patpos);
-		block_free(b);
-		block_set_note(song->blocks[editor->block],
-			       song->blocks[editor->block]->length-1,
-			       tracker->cursor_ch, 0, 0, 0);
-		block_set_command_full(song->blocks[editor->block],
-				       song->blocks[editor->block]->length-1,
-				       tracker->cursor_ch,
-				       editor->cmdpage, 0, 0);
-		tracker_redraw(tracker);
-	      }
-	      handled=TRUE;
-	      break;
 	    }
 	    if(data!=-1) {
 	      /* Set note and refresh */
@@ -580,9 +652,12 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	case 'p':
 	  note=28;
 	  break;
-	case '':
+	case 229:
 	  note=29;
 	  break;
+	case 180:
+	  note=30;
+	  break;
 	case GDK_Down:
 	  /* Down: Go down */
 	  editor_set_line(editor, editor->line+1);
@@ -629,6 +704,29 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  editor_set_line(editor, editor->line-8);
 	  handled=TRUE;
 	  break;
+	case GDK_BackSpace:
+	  {
+	    /* Backspace: delete line */
+	    int i;
+	    struct block *b=block_copy(song->blocks[editor->block],
+				       tracker->cursor_ch,
+				       tracker->patpos+1,
+				       tracker->cursor_ch,
+				       song->blocks[editor->block]->length-1);
+	    block_paste(song->blocks[editor->block], b,
+			tracker->cursor_ch, tracker->patpos);
+	    block_free(b);
+	    block_set_note(song->blocks[editor->block],
+			   song->blocks[editor->block]->length-1,
+			   tracker->cursor_ch, 0, 0, 0);
+	    for(i=0; i<song->blocks[editor->block]->commandpages; i++)
+	      block_set_command_full(song->blocks[editor->block],
+				     song->blocks[editor->block]->length-1,
+				     tracker->cursor_ch, i, 0, 0);
+	    tracker_redraw(tracker);
+	  }
+	  handled=TRUE;
+	  break;
 	}
 
 	/* Select an instrument if an instrument selection key was pressed */
@@ -691,6 +789,8 @@ gboolean keyboard_event(GtkWidget *widget, GdkEventKey *event,
 	  case 'o':
 	  case '0':
 	  case 'p':
+	  case 229:
+	  case 180:
 	    tracker_step_cursor_channel(tracker, -1);
 	    editor->chordstatus--;
 	    if(editor->chordstatus==0)
