@@ -113,6 +113,7 @@ void player_play_note(struct player *player, unsigned int instrument, unsigned c
     note += song->instruments[instrument]->transpose;
 
     /* Set new track status (channel, note, volume, time) */
+    player->trackstatus[track * TRACK_STATUS_NUMVALUES + TRACK_STATUS_INSTRUMENT] = instrument;
     player->trackstatus[track * TRACK_STATUS_NUMVALUES + TRACK_STATUS_MIDI_CHANNEL] = song->instruments[instrument]->midichannel;
     player->trackstatus[track * TRACK_STATUS_NUMVALUES + TRACK_STATUS_NOTE] = note;
     player->trackstatus[track * TRACK_STATUS_NUMVALUES + TRACK_STATUS_VOLUME] = song->instruments[instrument]->defaultvelocity * volume / 127 * (song->trackvolumes[track] & 127) / 127 * ((255 - song->trackvolumes[track]) & 128) / 128 * song->mastervolume / 127;
@@ -471,12 +472,18 @@ static void player_thread(struct player *player)
       if (note != 0 && ((delay >= 0 && player->tick == delay) || (delay < 0 && player->tick % (-delay) == 0))) {
 	note--;
 
-	player_play_note(player, instrument - 1, note, volume, i);
+	/* Use previous instrument if none defined */
+	if (instrument == 0)
+	  instrument = player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_INSTRUMENT] + 1;
+	
+	/* Play note if instrument is defined */
+	if (instrument != 0)
+	  player_play_note(player, instrument - 1, note, volume, i);
 
 	/* If no hold value was defined use the instrument's hold value */
 	if (hold == -1 && instrument <= song->numinstruments && song->instruments[instrument - 1] != NULL)
-	  hold = song->instruments[b->notes[(b->tracks * player->line + i) * 2 + 1] - 1]->hold;
-
+	  hold = song->instruments[instrument - 1]->hold;
+	
 	if (hold == 0)
 	  player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_HOLD] = -1;
 	else
@@ -739,6 +746,7 @@ void player_trackstatus_create(struct player *player, int oldmax)
 
   /* Set new tracks to -1 */
   for (i = oldmax; i < player->song->maxtracks; i++) {
+    player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_INSTRUMENT] = -1;
     player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_MIDI_CHANNEL] = -1;
     player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_NOTE] = -1;
     player->trackstatus[i * TRACK_STATUS_NUMVALUES + TRACK_STATUS_VOLUME] = -1;
